// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
// vi: set et ts=4 sw=2 sts=2:
#ifndef DUNE_CURVED_SURFACE_GRID_GRIDFUNCTION_HH
#define DUNE_CURVED_SURFACE_GRID_GRIDFUNCTION_HH

#include <type_traits>
#include <dune/curvedsurfacegrid/concepts.hh>
#include <dune/curvedsurfacegrid/gridfunctions/analyticgridfunction.hh>

namespace Dune
{
  namespace Impl
  {
    template< class ES, class = void >
    struct GridOf;

    template< class ES >
    struct GridOf<ES, std::void_t<typename ES::GridView>>
    {
      using type = typename ES::GridView::Grid;
    };

    template< class ES >
    struct GridOf<ES, std::void_t<typename ES::Grid>>
    {
      using type = typename ES::Grid;
    };

  } // end namespace Impl

  //! Type-Traits to extract the grid-type from a GridFunction `GF`
  template< class GF >
  using GridOf_t = typename Impl::GridOf<std::decay_t<decltype(std::declval<GF>().entitySet())>>::type;

  //! Conditionally define the GridFunction type.
  /**
   * If the type `GF` is a GridFunction on the grid `Grid`, use this type as GridFunction type,
   * Otherwise it is a functor and construct an \ref AnalyticGridFunction on the `Grid`.
   **/
  template< class Grid, class GF >
  using GridFunctionOf_t
    = std::conditional_t<Dune::Concept::isGridFunction<GF, Grid>(), GF, AnalyticGridFunction<Grid,GF>>;

} // end namespace Dune

#endif // DUNE_CURVED_SURFACE_GRID_GRIDFUNCTION_HH
