// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
// vi: set et ts=4 sw=2 sts=2:

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif
#include <iostream>

#include <dune/common/parallel/mpihelper.hh> // An initializer of MPI
#include <dune/foamgrid/foamgrid.hh>
#include <dune/functions/common/differentiablefunctionfromcallables.hh>
#include <dune/functions/common/functionconcepts.hh>
#include <dune/functions/gridfunctions/analyticgridviewfunction.hh>
#include <dune/grid/io/file/gmshreader.hh>

#define STR(s) STR_HELPER(s)
#define STR_HELPER(s) #s

int main(int argc, char** argv)
{
  using namespace Dune;
  MPIHelper::instance(argc, argv);

  using Grid = FoamGrid<2,3>;
  using GridView = typename Grid::LeafGridView;
  std::unique_ptr<Grid> grid = GmshReader<Grid>::read( STR(DUNE_GRID_PATH) "sphere_coarse.msh");

  using Signature = FieldVector<double,3>(FieldVector<double,3>);
  using LocalSignature = FieldVector<double,3>(FieldVector<double,2>);

  auto f = Functions::makeDifferentiableFunctionFromCallables(
    Functions::SignatureTag<Signature>{},
    [](auto const& x) { return FieldMatrix<double,3>(0); },
    [](auto const& x) { return FieldMatrix<double,3,3>(0); }
  );

  auto gf = Functions::makeAnalyticGridViewFunction(f, grid->leafGridView());
  using GF = decltype(gf);

  static_assert(Functions::Concept::isFunction<GF,Signature>(), "");
  static_assert(Functions::Concept::isDifferentiableFunction<GF,Signature>(), "");

  auto lf = localFunction(gf);
  using LF = decltype(lf);

  static_assert(Functions::Concept::isFunction<LF,LocalSignature>(), "");

  using EntitySet = Functions::GridViewEntitySet<GridView,0>;
  using LocalContext = typename EntitySet::Element;

  static_assert(Functions::Concept::isLocalFunction<LF,LocalSignature,LocalContext>(), "");
  static_assert(Functions::Concept::isDifferentiableFunction<LF,LocalSignature,Functions::LocalDerivativeTraits<EntitySet>::template Traits>(), "");
  static_assert(Functions::Concept::isDifferentiableLocalFunction<LF,LocalSignature,LocalContext,Functions::LocalDerivativeTraits<EntitySet>::template Traits>(), "");
}